/*
initrnd.c  Version 1.3.0
Initialize random number generator seed to date, time,
& clock ticks. Resulting seed is unpredictable.
Copyright (C) 2000-2010   dondalah721@yahoo.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/*
Usage:
    	unsigned char *sd;
    	sd = (unsigned char *) initrnd();
*/

/*
User subroutine calls:
	void rnd(unsigned char *sd);

System subroutine calls:
	times
	time
*/

#include <malloc.h>
#include <time.h>
#include <sys/times.h>
#define SEEDLEN 16

unsigned char *initrnd()
	{
	int i;
	void rnd();
	unsigned char *sd,*r;
	time_t now,*p;
	clock_t clk,*q;
	struct tms t;
	/* allocate memory for seed */
	sd = (unsigned char *) malloc(SEEDLEN+16);
	/* get clock ticks since boot */
	clk = times(&t);
	/* get date & time */
	time(&now);
	r = sd;
	/* initialize seed to zeros */
	for (i=0;i<SEEDLEN;i++) *r++ = '\0';
	/* fill 1st 8 bytes with data */
	p = &now;	/* date & time */
	q = &clk;	/* clock ticks @ 100/sec */
	r = sd;
	*r++ = *p++;
	*r++ = *p++;
	*r++ = *p++;
	*r++ = *p++;
	*r++ = *q++;
	*r++ = *q++;
	*r++ = *q++;
	*r++ = *q++;
	*sd |= 1;	/* seed must be odd */
	/* make seed random */
	for (i=0;i<4096;i++) rnd(sd);
	return(sd);
	} /* initrnd */
